--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    11:19:44 03/14/06
-- Design Name:    
-- Module Name:    control - rtl
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use work.user_pkg.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity control is
	 port(
	 	-- Input lines
      in_reset_n     : in  std_logic;
		in_user_clk		: in  std_logic;
      in_user_enable : in  std_logic;
      in_rt_ready    : in  std_logic;
      in_qdr_ready   : in  std_logic;

		-- Mem read control
		out_mem_read_enable : out std_logic;
		out_mem_read_index : out std_logic_vector(INDEX_BITS-1 downto 0);

		-- RW Writer control
		out_done			: out std_logic;
		out_ftr_ptr		: out std_logic_vector(39 downto 3); 	

		-- Memphy init control
		out_write_mem_index	: out std_logic_vector(INDEX_BITS-1 downto 0);
		out_write_mem_data 	: out std_logic_vector(63 downto 0);
		out_write_mem_strobe	: out std_logic;

		-- PWM init control
 		out_pwm_write_col_sel	: out std_logic_vector(PWM_COL_BITS-1 downto 0);
		out_pwm_write_row_sel	: out std_logic_vector(PWM_ROW_BITS-1 downto 0);
		out_pwm_write_data		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_pwm_write_strobe		: out std_logic;

		-- RT Fabric Request
    	freq_addr     	: in  std_logic_vector(39 downto 3);
    	freq_size    	: in  std_logic_vector(3 downto 0);
    	freq_srctag   	: in  std_logic_vector(4 downto 0);
    	freq_mask     	: in  std_logic_vector(7 downto 0);
    	freq_data     	: in  std_logic_vector(63 downto 0);
    	freq_rw_n     	: in  std_logic;
    	freq_ts       	: in  std_logic;
    	freq_valid    	: in  std_logic;
    	freq_enable   	: out std_logic;

    	-- RT User Response
    	uresp_full    	: in  std_logic;
    	uresp_ts      	: out std_logic;
    	uresp_size    	: out std_logic_vector(3 downto 0);
    	uresp_srctag  	: out std_logic_vector(4 downto 0);
    	uresp_data    	: out std_logic_vector(63 downto 0)

		--debug					: out std_logic_vector(15 downto 0)
	 );
end control;

architecture rtl of control is

	signal burst_ctrl : std_logic_vector(3 downto 0);
	signal burst_addr : std_logic_vector(27 downto 0);
	signal burst_data : std_logic_vector(63 downto 0);

	signal status_start	 	: std_logic;
	signal status_active 	: std_logic;
	signal status_done : std_logic;

	signal mem_start_index 	: std_logic_vector(INDEX_BITS-1 downto 0);
	signal mem_end_index		: std_logic_vector(INDEX_BITS-1 downto 0);
	signal mem_curr_index	: std_logic_vector(INDEX_BITS-1 downto 0);

	signal cooldown : std_logic_vector(4 downto 0);
	
	signal debug : std_logic_vector(63 downto 0);	   
begin

	uresp_size	<= (others => '0');
	uresp_srctag<= (others => '0');
	uresp_data	<= (others => '0');
	uresp_ts 	<= '0';

	rtread : process (in_user_clk, in_reset_n) is
	begin
		if in_reset_n='0' then
			freq_enable <= '0';

			status_start <= '0';

			mem_start_index <= (others => '0');
			mem_end_index <= (others => '0');

			burst_ctrl	<= (others => '0');
			burst_addr	<= (others => '0');
			burst_data	<= (others => '0');

	 		out_pwm_write_col_sel	<= (others => '0');
			out_pwm_write_row_sel	<= (others => '0');
			out_pwm_write_data		<= (others => '0');
			out_pwm_write_strobe		<= '0';

			out_write_mem_index	<= (others => '0');
			out_write_mem_data 	<= (others => '0');
			out_write_mem_strobe	<= '0';

			out_ftr_ptr <= (others => '0');

			out_mem_read_enable <= '0';
			out_mem_read_index <= (others => '0');
			out_done <= '0';
			status_done <= '0';
			status_active <= '0';

			mem_curr_index <= (others => '0');

			debug <= (others => '0');
		elsif in_user_clk='1' and in_user_clk'event then
			if status_start='1' then
				mem_curr_index <= mem_start_index;
				cooldown <= (others => '0');
				status_active <= '1';
				status_start <= '0';
			elsif status_active='1' then
				if mem_curr_index <= mem_end_index then
					out_mem_read_index <= mem_curr_index;
					mem_curr_index <= mem_curr_index + 1;
					out_mem_read_enable <= '1';

					debug(7 downto 5) <= "001";
				elsif cooldown <= PIPELINE_COOLDOWN then
					out_mem_read_index <= mem_curr_index;
					mem_curr_index <= mem_curr_index + 1;
					out_mem_read_enable <= '0';
					cooldown <= cooldown + 1;

					debug(7 downto 5) <= "010";
				elsif status_done='0' then
					out_done <= '1';
					status_done <= '1';

					debug(7 downto 5) <= "011";
				else
					status_active <= '0';
					out_done <= '0';
					status_done <= '0';

					debug(7 downto 5) <= "100";
				end if;
			end if;

			if in_qdr_ready='0' or in_rt_ready='0' then
				freq_enable <= '0';
				debug(4) <= '0';
			else
				freq_enable <= '1';
				debug(4) <= '1';

				if (freq_valid='1' and freq_ts='1') or (burst_ctrl > "0000") then
					if freq_rw_n='0' then

						burst_data <= freq_data;

						if burst_ctrl > "0000" then
							burst_ctrl <= burst_ctrl - 1;
							burst_addr <= burst_addr + 8;

							debug(3) <= '1';

							if burst_addr(27)='1' or burst_addr(26)='1' then -- mem write
								debug(2 downto 0) <= "100";

								out_write_mem_index	<= burst_addr(INDEX_BITS-1 downto 0);
								out_write_mem_data	<= burst_data;
								out_write_mem_strobe	<= '1';
								out_pwm_write_strobe <= '0';
							elsif burst_addr(25)='1' then -- pwm write
								debug(2 downto 0) <= "101";

								out_pwm_write_col_sel <= burst_addr(PWM_COL_BITS+2 downto 3);
								out_pwm_write_row_sel <= burst_addr(PWM_ROW_BITS+PWM_COL_BITS+2 downto PWM_COL_BITS+3);
								out_pwm_write_data <= burst_data(RESOLUTION_INPUT-1 downto 0);
								out_pwm_write_strobe <= '1';
								out_write_mem_strobe	<= '0';
							else -- settings and control
								out_pwm_write_strobe <= '0';
								out_write_mem_strobe <= '0';

								case burst_addr(6 downto 3) is
									when X"4" => -- 0x20
										debug(2 downto 0) <= "001";
										mem_start_index <= burst_data(INDEX_BITS-1 downto 0);
									when X"5" => -- 0x28
										debug(2 downto 0) <= "010";
										mem_end_index <= burst_data(INDEX_BITS-1 downto 0);
									when X"6" => -- 0x30
										debug(2 downto 0) <= "011";
										out_ftr_ptr <= burst_data(39 downto 3);
									when X"F" => -- 0x78
										debug(2 downto 0) <= "000";
										status_start <= '1';
									when others =>
										null;
								end case;
							end if;
						else
							debug(3) <= '0';

							out_pwm_write_strobe <= '0';
							out_write_mem_strobe <= '0';
							burst_ctrl <= freq_size + 1;
							burst_addr(26 downto 3) <= freq_addr(26 downto 3);
						end if;		
					else
						debug(3) <= '0';
					end if;
				else
					debug(3) <= '0';

					out_pwm_write_strobe <= '0';
					out_write_mem_strobe <= '0';
				end if;
			end if;
		end if;
	end process rtread;

--	ctrl : process (in_user_clk, in_reset_n, status_start, mem_end_index) is
--	begin
--		if in_reset_n='0' then
----			out_mem_read_enable <= '0';
----			out_mem_read_index <= (others => '0');
----			out_done <= '0';
----			local_done <= '0';
----			status_active <= '0';
----
----			mem_curr_index <= (others => '0');
--		elsif in_user_clk='1' and in_user_clk'event then
----			if status_start='1' then
----				mem_curr_index <= mem_start_index;
----				cooldown <= (others => '0');
----				status_active <= '1';
----			elsif status_active='1' then
----				if mem_curr_index <= mem_end_index then
----					out_mem_read_index <= mem_curr_index;
----					mem_curr_index <= mem_curr_index + 1;
----					out_mem_read_enable <= '1';
----				elsif cooldown <= PIPELINE_COOLDOWN then
----					out_mem_read_index <= (others => '0');
----					out_mem_read_enable <= '0';
----					cooldown <= cooldown + 1;
----				elsif local_done='0' then
----					out_done <= '1';
----					local_done <= '1';
----				else
----					status_active <= '0';
----					out_done <= '0';
----					local_done <= '0';
----				end if;
----			end if;
--		end if;
--	end process ctrl;
--
end rtl;
