--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    10:14:44 03/12/06
-- Design Name:    
-- Module Name:    memphy - rtl
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use work.user_pkg.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity memphy is
	 port(
		-- Global signals
		in_user_clk				: in  std_logic;
		in_reset_n				: in  std_logic;

		mem_read_enable		: in  std_logic;

		-- Configuration lines
		in_write_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);
		in_write_data			: in  std_logic_vector(63 downto 0);
		in_write_strobe 		: in  std_logic;

		 -- RAM 1
		dr_1          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_1          : out std_logic_vector(19 downto 0);  -- read address
		aw_1          : out std_logic_vector(19 downto 0);  -- write address
		r_n_1         : out std_logic;      -- Read strobe
		w_n_1         : out std_logic;      -- Write strobe
		bw_n_1        : out std_logic_vector(7 downto 0);   -- byte write
		dw_1          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 2
		dr_2          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_2          : out std_logic_vector(19 downto 0);  -- read address
		aw_2          : out std_logic_vector(19 downto 0);  -- write address
		r_n_2         : out std_logic;      -- Read strobe
		w_n_2         : out std_logic;      -- Write strobe
		bw_n_2        : out std_logic_vector(7 downto 0);   -- byte write
		dw_2          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 3
		dr_3          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_3          : out std_logic_vector(19 downto 0);  -- read address
		aw_3          : out std_logic_vector(19 downto 0);  -- write address
		r_n_3         : out std_logic;      -- Read strobe
		w_n_3         : out std_logic;      -- Write strobe
		bw_n_3        : out std_logic_vector(7 downto 0);   -- byte write
		dw_3          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 4
		dr_4          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_4          : out std_logic_vector(19 downto 0);  -- read address
		aw_4          : out std_logic_vector(19 downto 0);  -- write address
		r_n_4         : out std_logic;      -- Read strobe
		w_n_4         : out std_logic;      -- Write strobe
		bw_n_4        : out std_logic_vector(7 downto 0);   -- byte write
		dw_4          : out std_logic_vector(71 downto 0);  -- write data
		
		-- Memread lines
		in_index		 	: in  std_logic_vector(INDEX_BITS-1 downto 0);
		out_data			: out std_logic_vector(PWM_ROW_BITS downto 0)		-- element from memory store
		--out_index		: out std_logic_vector(INDEX_BITS-1 downto 0)

		--debug				: out std_logic_vector(63 downto 0)
	 );
end memphy;

architecture rtl of memphy is

	-- Delayed index shift registers	(8-clock latency)
--	signal 	index_0, index_1, index_2, index_3, 
--				index_4, index_5, index_6, index_7 : std_logic_vector(INDEX_BITS-1 downto 0);

	signal read_fsm_0 : std_logic_vector(2 downto 0); -- 8 states, one for each byte of the quadword

	signal debug : std_logic_vector(63 downto 0);
begin

--	shift_indexes : process (in_user_clk, in_reset_n) is
--	begin
--		if in_reset_n='0' then
--			index_0	<= (others => '0');
--			index_1	<= (others => '0');
--			index_2	<= (others => '0');
--			index_3	<= (others => '0');
--			index_4	<= (others => '0');
--			index_5	<= (others => '0');
--			index_6	<= (others => '0');
--			index_7	<= (others => '0');
--			debug(63 downto 27) <= (others => '0');
--	 	elsif in_user_clk='1' and in_user_clk'event then
--			index_0	<= index_1;
--			index_1	<= index_2;
--			index_2	<= index_3;
--			index_3	<= index_4;
--			index_4	<= index_5;
--			index_5	<= index_6;
--			index_6	<= index_7;
--
--			if mem_read_enable='1' then
--				index_7	<= in_index;
--			else
--				index_7 <= (others => '1');
--			end if;
--
--			debug(44 downto 27) <= index_0(17 downto 0);
--			debug(62 downto 45) <= in_index(17 downto 0);
--		end if;
--	end process shift_indexes; 
--

	mem_read : process (in_user_clk, in_reset_n) is
	begin
		if in_reset_n='0' then
			out_data		<= (others => '1');
			--out_index	<= (others => '0');
			read_fsm_0	<= (others => '0');

			ar_1			<=	(others => '0');
			r_n_1			<= '1';

			ar_2			<=	(others => '0');
			r_n_2			<= '1';

			ar_3			<=	(others => '0');
			r_n_3			<= '1';

			ar_4			<=	(others => '0');
			r_n_4			<= '1';

			debug(26 downto 0) <= (others => '0');
		elsif in_user_clk='1' and in_user_clk'event then
			if mem_read_enable='1' then
				r_n_1 <= '0';
				ar_1(19 downto 0) <= in_index(22 downto 3);
				read_fsm_0 <= read_fsm_0 + 1;

				if read_fsm_0 = "000" then
				  	out_data		<= dr_1(2 downto 0); 
					--read_fsm_0 	<= "001";
					--debug(26 downto 24) <= "001";
				elsif read_fsm_0 = "001" then
					out_data		<= dr_1(11 downto 9);
					--read_fsm_0 	<= "010";
					--debug(26 downto 24) <= "010";
				elsif read_fsm_0 = "010" then
					out_data		<= dr_1(20 downto 18);
					--read_fsm_0 	<= "011";
					--debug(26 downto 24) <= "011";
				elsif read_fsm_0 = "011" then
					out_data		<= dr_1(29 downto 27);
					--read_fsm_0 	<= "100";
					--debug(26 downto 24) <= "100";
				elsif read_fsm_0 = "100" then
					out_data		<= dr_1(38 downto 36);
					--read_fsm_0 	<= "101";
					--debug(26 downto 24) <= "101";
				elsif read_fsm_0 = "101" then
					out_data		<= dr_1(47 downto 45);
					--read_fsm_0 	<= "110";
					--debug(26 downto 24) <= "110";
				elsif read_fsm_0 = "110" then
					out_data		<= dr_1(56 downto 54);
					--read_fsm_0 	<= "111";
					--debug(26 downto 24) <= "111";
				elsif read_fsm_0 = "111" then
					out_data		<= dr_1(65 downto 63);
					--read_fsm_0 	<= "000";
					--debug(26 downto 24) <= "000";
				end if;
			else
				ar_1 <= (others => '0');
				r_n_1 <= '1';
				out_data <= (others => '1');
			end if;	

			debug(2 downto 0) <= dr_1(2 downto 0);
			debug(5 downto 3) <= dr_1(11 downto 9);
			debug(8 downto 6) <= dr_1(20 downto 18);
			debug(11 downto 9) <= dr_1(29 downto 27);
			debug(14 downto 12) <= dr_1(38 downto 36);
			debug(17 downto 15) <= dr_1(47 downto 45);
			debug(20 downto 18) <= dr_1(56 downto 54);
			debug(23 downto 21) <= dr_1(65 downto 63);

			--out_index <= index_0;
		end if;
	end process mem_read;

	mem_write : process (in_user_clk, in_reset_n) is
	begin
 		if in_reset_n='0' then
			aw_1			<= (others => '0');
			w_n_1			<= '1';
			bw_n_1		<= (others => '0');
			dw_1			<= (others => '0');

			aw_2			<= (others => '0');
			w_n_2			<= '1';
			bw_n_2		<= (others => '0');
			dw_2			<= (others => '0');

			aw_3			<= (others => '0');
			w_n_3			<= '1';
			bw_n_3		<= (others => '0');
			dw_3			<= (others => '0');

			aw_4			<= (others => '0');
			w_n_4			<= '1';
			bw_n_4		<= (others => '0');
			dw_4			<= (others => '0');
		elsif in_user_clk='1' and in_user_clk'event then
			if in_write_strobe='1' then
				aw_1(19 downto 0) <= in_write_index(22 downto 3);

				dw_1(7 downto 0)	 <= in_write_data(7 downto 0);
				dw_1(16 downto 9)	 <= in_write_data(15 downto 8);
				dw_1(25 downto 18) <= in_write_data(23 downto 16);
				dw_1(34 downto 27) <= in_write_data(31 downto 24);
				dw_1(43 downto 36) <= in_write_data(39 downto 32);
				dw_1(52 downto 45) <= in_write_data(47 downto 40);
				dw_1(61 downto 54) <= in_write_data(55 downto 48);
				dw_1(70 downto 63) <= in_write_data(63 downto 56);

				w_n_1 <= '0';
			else
				aw_1 <= (others => '0');
				dw_1 <= (others => '0');
				w_n_1 <= '1';
			end if;
		end if;

	end process mem_write;

end rtl;
