--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    05:44:56 03/06/06
-- Design Name:    
-- Module Name:    pwm - rtl
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;
use work.user_pkg.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity pwm is
	port (		
		-- Global input signals
      in_reset_n     : in  std_logic;
		in_user_clk		: in  std_logic;

	 	in_reg_col_sel	: in  std_logic_vector(PWM_COL_BITS-1 downto 0);		-- PWM cols	(window length)
		in_reg_row_sel	: in  std_logic_vector(PWM_ROW_BITS-1 downto 0);  		-- PWM rows (number of elements)
		in_reg_data		: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);	-- Input value resolution
		in_reg_strobe	: in  std_logic;

		-- Input lines
		in_elem			: in  std_logic_vector(PWM_ROW_BITS downto 0);
		--in_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);

		-- Output lines
		out_data_00		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_01		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_02		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_03		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_04		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_05		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_06		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_07		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0)
		--out_index		: out std_logic_vector(INDEX_BITS-1 downto 0)

		--debug				: out std_logic_vector(63 downto 0)
	);
end pwm;

architecture rtl of pwm is

	signal elem_00 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_01 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_02 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_03 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_04 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_05 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_06 	: std_logic_vector(PWM_ROW_BITS downto 0);
	signal elem_07 	: std_logic_vector(PWM_ROW_BITS downto 0);

--	signal index_00	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_01	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_02	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_03	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_04	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_05	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_06	: std_logic_vector(INDEX_BITS-1 downto 0);
--	signal index_07	: std_logic_vector(INDEX_BITS-1 downto 0);

	signal matrix 		: PWM_mem;

	signal debug : std_logic_vector(63 downto 0);
begin
	set_pwm		: process (in_user_clk) is
	begin
  		if in_user_clk='1' and in_user_clk'event then
			if in_reg_strobe='1' then
				--matrix(to_integer(unsigned(in_reg_col_sel)), to_integer(unsigned(in_reg_row_sel))) <= in_reg_data;
				case in_reg_row_sel is
				when "00"	=>
					case in_reg_col_sel is 					 
					when "000"	=>
						matrix(0,0)	<= in_reg_data;
					when "001"	=>
						matrix(1,0)	<= in_reg_data;
					when "010"	=>
						matrix(2,0)	<= in_reg_data;
					when "011"	=>
						matrix(3,0)	<= in_reg_data;
					when "100"	=>
						matrix(4,0)	<= in_reg_data;
					when "101"	=>
						matrix(5,0)	<= in_reg_data;
					when "110"	=>
						matrix(6,0)	<= in_reg_data;
					when "111"	=>
						matrix(7,0)	<= in_reg_data;
					when others	=>
						null;
					end case; 
				when "01"	=>
					case in_reg_col_sel is 					 
					when "000"	=>
						matrix(0,1)	<= in_reg_data;
					when "001"	=>
						matrix(1,1)	<= in_reg_data;
					when "010"	=>
						matrix(2,1)	<= in_reg_data;
					when "011"	=>
						matrix(3,1)	<= in_reg_data;
					when "100"	=>
						matrix(4,1)	<= in_reg_data;
					when "101"	=>
						matrix(5,1)	<= in_reg_data;
					when "110"	=>
						matrix(6,1)	<= in_reg_data;
					when "111"	=>
						matrix(7,1)	<= in_reg_data;
					when others	=>
						null;
					end case; 
				when "10"	=>
					case in_reg_col_sel is 					 
					when "000"	=>
						matrix(0,2)	<= in_reg_data;
					when "001"	=>
						matrix(1,2)	<= in_reg_data;
					when "010"	=>
						matrix(2,2)	<= in_reg_data;
					when "011"	=>
						matrix(3,2)	<= in_reg_data;
					when "100"	=>
						matrix(4,2)	<= in_reg_data;
					when "101"	=>
						matrix(5,2)	<= in_reg_data;
					when "110"	=>
						matrix(6,2)	<= in_reg_data;
					when "111"	=>
						matrix(7,2)	<= in_reg_data;
					when others	=>
						null;
					end case; 
				when "11"	=>
					case in_reg_col_sel is 					 
					when "000"	=>
						matrix(0,3)	<= in_reg_data;
					when "001"	=>
						matrix(1,3)	<= in_reg_data;
					when "010"	=>
						matrix(2,3)	<= in_reg_data;
					when "011"	=>
						matrix(3,3)	<= in_reg_data;
					when "100"	=>
						matrix(4,3)	<= in_reg_data;
					when "101"	=>
						matrix(5,3)	<= in_reg_data;
					when "110"	=>
						matrix(6,3)	<= in_reg_data;
					when "111"	=>
						matrix(7,3)	<= in_reg_data;
					when others	=>
						null;
					end case; 
				when others	=>
					null; 
				end case; 		
			end if;
		end if; 
	end process set_pwm;

	shift_elems : process (in_user_clk, in_reset_n) is
	begin
	 	if in_reset_n='0' then
			elem_00	<= (others => '1');
			elem_01	<= (others => '1');
			elem_02	<= (others => '1');
			elem_03	<= (others => '1');
			elem_04	<= (others => '1');
			elem_05	<= (others => '1');
			elem_06	<= (others => '1');
			elem_07	<= (others => '1');

--			index_00 <= (others => '0');
--			index_01 <= (others => '0');
--			index_02 <= (others => '0');
--			index_03 <= (others => '0');
--			index_04 <= (others => '0');
--			index_05 <= (others => '0');
--			index_06 <= (others => '0');
--			index_07 <= (others => '0');

			debug(31 downto 0) <= (others => '0');
	 	elsif in_user_clk='1' and in_user_clk'event then
			elem_00	<= elem_01;
			elem_01	<= elem_02;
			elem_02	<= elem_03;
			elem_03	<= elem_04;
			elem_04	<= elem_05;
			elem_05	<= elem_06;
			elem_06	<= elem_07;
			elem_07	<= in_elem;

--			index_00	<= index_01;
--			index_01	<= index_02;
--			index_02	<= index_03;
--			index_03	<= index_04;
--			index_04	<= index_05;
--			index_05	<= index_06;
--			index_06	<= index_07;
--			index_07	<= in_index;

			debug(2 downto 0) <= elem_00;
			debug(5 downto 3) <= elem_01;
			debug(8 downto 6) <= elem_02;
			debug(11 downto 9) <= elem_03;
			debug(14 downto 12) <= elem_04;
			debug(17 downto 15) <= elem_05;
			debug(20 downto 18) <= elem_06;
			debug(23 downto 21) <= elem_07;
			--debug(INDEX_BITS+23 downto 24) <= index_00;
		end if;
	end process shift_elems; 

	match : process (in_user_clk, in_reset_n) is
	begin
	 	if in_reset_n='0' then
			out_data_00	<= (others => '0');
			out_data_01	<= (others => '0');
			out_data_02	<= (others => '0');
			out_data_03	<= (others => '0');
			out_data_04	<= (others => '0');
			out_data_05	<= (others => '0');
			out_data_06	<= (others => '0');
			out_data_07	<= (others => '0');
--			out_index	<= (others => '0');
			debug(63 downto 32) <= (others => '0');
 	 	elsif in_user_clk='1' and in_user_clk'event then
--			out_data_00 <= matrix(to_integer(unsigned(elem_00)), 0);
--			out_data_01 <= matrix(to_integer(unsigned(elem_01)), 0);
--			out_data_02 <= matrix(to_integer(unsigned(elem_02)), 0);
--			out_data_03 <= matrix(to_integer(unsigned(elem_03)), 0);
--			out_data_04 <= matrix(to_integer(unsigned(elem_04)), 0);
--			out_data_05 <= matrix(to_integer(unsigned(elem_05)), 0);
--			out_data_06 <= matrix(to_integer(unsigned(elem_06)), 0);
--			out_data_07 <= matrix(to_integer(unsigned(elem_07)), 0);
--
			if (elem_00(PWM_ROW_BITS) or elem_01(PWM_ROW_BITS) or 
				 elem_02(PWM_ROW_BITS) or elem_03(PWM_ROW_BITS) or 
				 elem_04(PWM_ROW_BITS) or elem_05(PWM_ROW_BITS) or 
				 elem_06(PWM_ROW_BITS) or elem_07(PWM_ROW_BITS) ) = '1' then
				out_data_00	<= (others => '0');
				out_data_01	<= (others => '0');
				out_data_02	<= (others => '0');
				out_data_03	<= (others => '0');
				out_data_04	<= (others => '0');
				out_data_05	<= (others => '0');
				out_data_06	<= (others => '0');
				out_data_07	<= (others => '0');
--				out_index 	<= (others => '0');
				debug(63 downto 32) <= (others => '0');
			else
				case elem_00 is
					when "000"	=>
						out_data_00 <= matrix(0,0);
						debug(35 downto 32) <= matrix(0,0)(3 downto 0); 
					when "001"	=>
						out_data_00 <= matrix(0,1); 
						debug(35 downto 32) <= matrix(0,1)(3 downto 0);
					when "010"	=>
						out_data_00 <= matrix(0,2); 
						debug(35 downto 32) <= matrix(0,2)(3 downto 0);
					when "011"	=>
						out_data_00 <= matrix(0,3);
						debug(35 downto 32) <= matrix(0,3)(3 downto 0);
					when others	=>
						out_data_00 <= (others => '0');
						debug(35 downto 32) <= (others => '0'); 
				end case; 

				case elem_01 is
					when "000"	=>
						out_data_01 <= matrix(1,0); 
						debug(39 downto 36) <= matrix(1,0)(3 downto 0);
					when "001"	=>
						out_data_01 <= matrix(1,1); 
						debug(39 downto 36) <= matrix(1,1)(3 downto 0);
					when "010"	=>
						out_data_01 <= matrix(1,2); 
						debug(39 downto 36) <= matrix(1,2)(3 downto 0);
					when "011"	=>
						out_data_01 <= matrix(1,3);
						debug(39 downto 36) <= matrix(1,3)(3 downto 0);
					when others	=>
						out_data_01 <= (others => '0');
						debug(39 downto 36) <= (others => '0'); 
				end case; 

				case elem_02 is
					when "000"	=>
						out_data_02 <= matrix(2,0); 
						debug(43 downto 40) <= matrix(2,0)(3 downto 0);
					when "001"	=>
						out_data_02 <= matrix(2,1); 
						debug(43 downto 40) <= matrix(2,1)(3 downto 0);
					when "010"	=>
						out_data_02 <= matrix(2,2); 
						debug(43 downto 40) <= matrix(2,2)(3 downto 0);
					when "011"	=>					 
						out_data_02 <= matrix(2,3);
						debug(43 downto 40) <= matrix(2,3)(3 downto 0);
					when others	=>
						out_data_02 <= (others => '0');
						debug(43 downto 40) <= (others => '0'); 
				end case; 

				case elem_03 is
					when "000"	=>
						out_data_03 <= matrix(3,0); 
						debug(47 downto 44) <= matrix(3,0)(3 downto 0);
					when "001"	=>
						out_data_03 <= matrix(3,1); 
						debug(47 downto 44) <= matrix(3,1)(3 downto 0);
					when "010"	=>
						out_data_03 <= matrix(3,2); 
						debug(47 downto 44) <= matrix(3,2)(3 downto 0);
					when "011"	=>
						out_data_03 <= matrix(3,3);
						debug(47 downto 44) <= matrix(3,3)(3 downto 0);
					when others	=>
						out_data_03 <= (others => '0');
						debug(47 downto 44) <= (others => '0'); 
				end case; 

				case elem_04 is
					when "000"	=>
						out_data_04 <= matrix(4,0); 
						debug(51 downto 48) <= matrix(4,0)(3 downto 0);
					when "001"	=>
						out_data_04 <= matrix(4,1); 
						debug(51 downto 48) <= matrix(4,1)(3 downto 0);
					when "010"	=>
						out_data_04 <= matrix(4,2); 
						debug(51 downto 48) <= matrix(4,2)(3 downto 0);
					when "011"	=>
						out_data_04 <= matrix(4,3);
						debug(51 downto 48) <= matrix(4,3)(3 downto 0);
					when others	=>
						out_data_04 <= (others => '0');
						debug(51 downto 48) <= (others => '0'); 
				end case; 

				case elem_05 is
					when "000"	=>
						out_data_05 <= matrix(5,0); 
						debug(55 downto 52) <= matrix(5,0)(3 downto 0);
					when "001"	=>
						out_data_05 <= matrix(5,1); 
						debug(55 downto 52) <= matrix(5,1)(3 downto 0);
					when "010"	=>
						out_data_05 <= matrix(5,2); 
						debug(55 downto 52) <= matrix(5,2)(3 downto 0);
					when "011"	=>
						out_data_05 <= matrix(5,3);
						debug(55 downto 52) <= matrix(5,3)(3 downto 0);
					when others	=>
						out_data_05 <= (others => '0');
						debug(55 downto 52) <= (others => '0'); 
				end case; 

				case elem_06 is
					when "000"	=>
						out_data_06 <= matrix(6,0); 
						debug(59 downto 56) <= matrix(6,0)(3 downto 0);
					when "001"	=>
						out_data_06 <= matrix(6,1); 
						debug(59 downto 56) <= matrix(6,1)(3 downto 0);
					when "010"	=>
						out_data_06 <= matrix(6,2); 
						debug(59 downto 56) <= matrix(6,2)(3 downto 0);
					when "011"	=>
						out_data_06 <= matrix(6,3);
						debug(59 downto 56) <= matrix(6,3)(3 downto 0);
					when others	=>
						out_data_06 <= (others => '0');
						debug(59 downto 56) <= (others => '0'); 
				end case; 

				case elem_07 is
					when "000"	=>
						out_data_07 <= matrix(7,0); 
						debug(63 downto 60) <= matrix(7,0)(3 downto 0);
					when "001"	=>
						out_data_07 <= matrix(7,1); 
						debug(63 downto 60) <= matrix(7,1)(3 downto 0);
					when "010"	=>
						out_data_07 <= matrix(7,2); 
						debug(63 downto 60) <= matrix(7,2)(3 downto 0);
					when "011"	=>
						out_data_07 <= matrix(7,3);
						debug(63 downto 60) <= matrix(7,3)(3 downto 0);
					when others	=>
						out_data_07 <= (others => '0');
						debug(63 downto 60) <= (others => '0'); 
				end case;
				
--				debug(34 downto 32) <= elem_00;
--				debug(37 downto 35) <= elem_01;
--				debug(40 downto 38) <= elem_02;
--				debug(43 downto 41) <= elem_03;
--				debug(46 downto 44) <= elem_04;
--				debug(49 downto 47) <= elem_05;
--				debug(52 downto 50) <= elem_06;
--				debug(55 downto 53) <= elem_07;
-- 
 
--				out_index <= index_00;
			end if;
		end if;
	end process match;
end rtl;
