-------------------------------------------------------------------------------
-- Title      : User Application Block
-- Project    : VHDL Template FPGA
-------------------------------------------------------------------------------
-- File       : $RCSfile: user_app.vhd,v $
-- Author     : Cray Canada
-- Company    : Cray Canada Inc.
-- Created    : 2005-01-04
-- Last update: 2005-02-25
-------------------------------------------------------------------------------
-- Description: This block is the top level file of the user application logic.
-- It is instantiated by the 'top.vhd' file.  All design specific code should
-- be contained in this file and any sub blocks it instantiates.
-------------------------------------------------------------------------------
-- Copyright (c) 2005 Cray Canada Inc.
-------------------------------------------------------------------------------
-- Revisions  :
-- $Log: user_app.vhd,v $
-- Revision 1.2  2005/02/26 00:12:46 
-- Cleaned up naming.
--
-- Revision 1.1  2005/01/04 20:53:11 
-- Initial checkin.
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use work.user_pkg.all;

entity user_app is
 
  port (
    -- Global signals
    reset_n       : in  std_logic;
    user_clk      : in  std_logic;
    user_enable   : in  std_logic;
    rt_ready      : in  std_logic;
    qdr_ready     : in  std_logic;
    -- QDR II RAM Interface
    -- RAM 1
    dr_1          : in  std_logic_vector(71 downto 0);  -- read data 
    ar_1          : out std_logic_vector(19 downto 0);  -- read address
    aw_1          : out std_logic_vector(19 downto 0);  -- write address
    r_n_1         : out std_logic;      -- Read strobe
    w_n_1         : out std_logic;      -- Write strobe
    bw_n_1        : out std_logic_vector(7 downto 0);   -- byte write
    dw_1          : out std_logic_vector(71 downto 0);  -- write data
    -- RAM 2
    dr_2          : in  std_logic_vector(71 downto 0);  -- read data 
    ar_2          : out std_logic_vector(19 downto 0);  -- read address
    aw_2          : out std_logic_vector(19 downto 0);  -- write address
    r_n_2         : out std_logic;      -- Read strobe
    w_n_2         : out std_logic;      -- Write strobe
    bw_n_2        : out std_logic_vector(7 downto 0);   -- byte write
    dw_2          : out std_logic_vector(71 downto 0);  -- write data
    -- RAM 3
    dr_3          : in  std_logic_vector(71 downto 0);  -- read data 
    ar_3          : out std_logic_vector(19 downto 0);  -- read address
    aw_3          : out std_logic_vector(19 downto 0);  -- write address
    r_n_3         : out std_logic;      -- Read strobe
    w_n_3         : out std_logic;      -- Write strobe
    bw_n_3        : out std_logic_vector(7 downto 0);   -- byte write
    dw_3          : out std_logic_vector(71 downto 0);  -- write data
    -- RAM 4
    dr_4          : in  std_logic_vector(71 downto 0);  -- read data 
    ar_4          : out std_logic_vector(19 downto 0);  -- read address
    aw_4          : out std_logic_vector(19 downto 0);  -- write address
    r_n_4         : out std_logic;      -- Read strobe
    w_n_4         : out std_logic;      -- Write strobe
    bw_n_4        : out std_logic_vector(7 downto 0);   -- byte write
    dw_4          : out std_logic_vector(71 downto 0);  -- write data
    -- RT Interface
    -- User Request Interface
    ureq_srctag   : in  std_logic_vector(4 downto 0);   -- request srctag
    ureq_full     : in  std_logic;      -- request buffer full
    ureq_notag    : in  std_logic;      -- request out of source tags
    ureq_addr     : out std_logic_vector(39 downto 3);  -- request address
    ureq_size     : out std_logic_vector(2 downto 0);   -- request size
    ureq_mask     : out std_logic_vector(7 downto 0);   -- request byte mask
    ureq_rw_n     : out std_logic;      -- request read/write
    ureq_data     : out std_logic_vector(63 downto 0);  -- request write data
    ureq_ts       : out std_logic;      -- request transfer start
    ureq_byte_req : out std_logic;      -- request type (byte or double word)
    -- Fabric Response Interface
    fresp_valid   : in  std_logic;      -- response signals valid
    fresp_ts      : in  std_logic;      -- response transfer start
    fresp_size    : in  std_logic_vector(2 downto 0);   -- response size
    fresp_srctag  : in  std_logic_vector(4 downto 0);   -- response srctag
    fresp_data    : in  std_logic_vector(63 downto 0);  -- response data
    fresp_enable  : out std_logic;      -- enable responses
    -- Fabric Request Interface
    freq_addr     : in  std_logic_vector(39 downto 3);  -- request address
    freq_size     : in  std_logic_vector(3 downto 0);   -- request size
    freq_srctag   : in  std_logic_vector(4 downto 0);   -- request srctag
    freq_mask     : in  std_logic_vector(7 downto 0);   -- request byte mask
    freq_rw_n     : in  std_logic;      -- request read/write 
    freq_ts       : in  std_logic;      -- request transfer start
    freq_valid    : in  std_logic;      -- request valid
    freq_data     : in  std_logic_vector(63 downto 0);  -- request write data
    freq_enable   : out std_logic;      -- enable request interface
    -- User Response Interface
    uresp_full    : in  std_logic;      -- response buffer full
    uresp_ts      : out std_logic;      -- response transfer start
    uresp_size    : out std_logic_vector(3 downto 0);   -- response size
    uresp_srctag  : out std_logic_vector(4 downto 0);   -- response srctag
    uresp_data    : out std_logic_vector(63 downto 0)   -- response data

	 --debug 			: out std_logic_vector(63 downto 0)
 );

end entity user_app;

architecture rtl of user_app is

	-- Main controller and RT handler
	component control is
	 port(
	 	-- Input lines
      in_reset_n     : in  std_logic;
		in_user_clk		: in  std_logic;
      in_user_enable : in  std_logic;
      in_rt_ready    : in  std_logic;
      in_qdr_ready   : in  std_logic;

		-- Mem read control
 		out_mem_read_enable : out std_logic;
		out_mem_read_index : out std_logic_vector(INDEX_BITS-1 downto 0);

		-- RW Writer control
		out_done			: out std_logic;
		out_ftr_ptr		: out std_logic_vector(39 downto 3); 	

		-- Memphy init control
		out_write_mem_index	: out std_logic_vector(INDEX_BITS-1 downto 0);
		out_write_mem_data 	: out std_logic_vector(63 downto 0);
		out_write_mem_strobe	: out std_logic;

		-- PWM init control
 		out_pwm_write_col_sel	: out std_logic_vector(PWM_COL_BITS-1 downto 0);
		out_pwm_write_row_sel	: out std_logic_vector(PWM_ROW_BITS-1 downto 0);
		out_pwm_write_data		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_pwm_write_strobe		: out std_logic;

		-- RT Fabric Request
    	freq_addr     	: in  std_logic_vector(39 downto 3);
    	freq_size    	: in  std_logic_vector(3 downto 0);
    	freq_srctag   	: in  std_logic_vector(4 downto 0);
    	freq_mask     	: in  std_logic_vector(7 downto 0);
    	freq_data     	: in  std_logic_vector(63 downto 0);
    	freq_rw_n     	: in  std_logic;
    	freq_ts       	: in  std_logic;
    	freq_valid    	: in  std_logic;
    	freq_enable   	: out std_logic;

    	-- RT User Response
    	uresp_full    	: in  std_logic;
    	uresp_ts      	: out std_logic;
    	uresp_size    	: out std_logic_vector(3 downto 0);
    	uresp_srctag  	: out std_logic_vector(4 downto 0);
    	uresp_data    	: out std_logic_vector(63 downto 0)

		--debug				: out std_logic_vector(15 downto 0)
	 );
	end component;


	-- Memory controller
	component memphy is
	 port(
		-- Global signals
		in_user_clk				: in  std_logic;
		in_reset_n				: in  std_logic;

		mem_read_enable		: in  std_logic;

		-- Configuration lines
		in_write_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);
		in_write_data			: in  std_logic_vector(63 downto 0);
		in_write_strobe 		: in  std_logic;

		 -- RAM 1
		dr_1          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_1          : out std_logic_vector(19 downto 0);  -- read address
		aw_1          : out std_logic_vector(19 downto 0);  -- write address
		r_n_1         : out std_logic;      -- Read strobe
		w_n_1         : out std_logic;      -- Write strobe
		bw_n_1        : out std_logic_vector(7 downto 0);   -- byte write
		dw_1          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 2
		dr_2          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_2          : out std_logic_vector(19 downto 0);  -- read address
		aw_2          : out std_logic_vector(19 downto 0);  -- write address
		r_n_2         : out std_logic;      -- Read strobe
		w_n_2         : out std_logic;      -- Write strobe
		bw_n_2        : out std_logic_vector(7 downto 0);   -- byte write
		dw_2          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 3
		dr_3          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_3          : out std_logic_vector(19 downto 0);  -- read address
		aw_3          : out std_logic_vector(19 downto 0);  -- write address
		r_n_3         : out std_logic;      -- Read strobe
		w_n_3         : out std_logic;      -- Write strobe
		bw_n_3        : out std_logic_vector(7 downto 0);   -- byte write
		dw_3          : out std_logic_vector(71 downto 0);  -- write data
		 -- RAM 4
		dr_4          : in  std_logic_vector(71 downto 0);  -- read data 
		ar_4          : out std_logic_vector(19 downto 0);  -- read address
		aw_4          : out std_logic_vector(19 downto 0);  -- write address
		r_n_4         : out std_logic;      -- Read strobe
		w_n_4         : out std_logic;      -- Write strobe
		bw_n_4        : out std_logic_vector(7 downto 0);   -- byte write
		dw_4          : out std_logic_vector(71 downto 0);  -- write data
		
		-- Memread lines
		in_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);
		out_data			: out std_logic_vector(PWM_ROW_BITS downto 0)		-- element from memory store
		--out_index		: out std_logic_vector(INDEX_BITS-1 downto 0)

	--	debug				: out std_logic_vector(63 downto 0)
	 );
	end component memphy;


	component pwm is
	 port(
      in_reset_n     : in  std_logic;
		in_user_clk		: in  std_logic;

	 	in_reg_col_sel	: in  std_logic_vector(PWM_COL_BITS-1 downto 0);		-- PWM cols	(window length)
		in_reg_row_sel	: in  std_logic_vector(PWM_ROW_BITS-1 downto 0);  		-- PWM rows (number of elements)
		in_reg_data		: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);	-- Input value resolution
		in_reg_strobe	: in  std_logic;

		-- Input lines
		in_elem			: in  std_logic_vector(PWM_ROW_BITS downto 0);
		--in_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);

		-- Output lines
		out_data_00		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_01		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_02		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_03		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_04		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_05		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_06		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		out_data_07		: out std_logic_vector(RESOLUTION_INPUT-1 downto 0)
		--out_index		: out std_logic_vector(INDEX_BITS-1 downto 0)

		--debug				: out std_logic_vector(63 downto 0)
	 );
	end component pwm;
	 

	component adder is
	 port(
		in_user_clk			: in  std_logic;
		in_reset_n			: in  std_logic;

		-- Data Inputs (generate PWM_COL_BITS data input lines...)
		in_data_00			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_01			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_02			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_03			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_04			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_05			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_06			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		in_data_07			: in  std_logic_vector(RESOLUTION_INPUT-1 downto 0);
		--in_index				: in  std_logic_vector(INDEX_BITS-1 downto 0);
		
		-- Output lines
		out_data				: out std_logic_vector(RESOLUTION_OUTPUT-1 downto 0)
		--out_index			: out std_logic_vector(INDEX_BITS-1 downto 0)

		--debug					: out std_logic_vector(63 downto 0)
	 );
	end component adder;


	component resqueue is
	 port(
		in_user_clk		: in  std_logic;
		in_reset_n		: in  std_logic;

	 	-- Input signals
		in_data			: in  std_logic_vector(RESOLUTION_OUTPUT-1 downto 0);
		in_index			: in  std_logic_vector(INDEX_BITS-1 downto 0);

	  	-- Flags to tell resqueue to write results to RT, and where
		in_done			: in  std_logic;
		in_ftr_ptr		: in  std_logic_vector(39 downto 3);

		-- RT user request interface
	   ureq_srctag   	: in  std_logic_vector(4 downto 0);
	   ureq_full     	: in  std_logic;
	   ureq_notag    	: in  std_logic;
	   ureq_addr     	: out std_logic_vector(39 downto 3);
	   ureq_size     	: out std_logic_vector(2 downto 0);
	   ureq_mask     	: out std_logic_vector(7 downto 0);
	   ureq_data     	: out std_logic_vector(63 downto 0);
	   ureq_rw_n     	: out std_logic;
	   ureq_ts       	: out std_logic;
	   ureq_byte_req 	: out std_logic

		--debug				: out std_logic_vector(63 downto 0)
	 );
	end component resqueue;


	-- Control signals
	signal mem_read_enable		: std_logic;
	signal res_save_enable		: std_logic;
	signal done						: std_logic;
	signal ftr_ptr					: std_logic_vector(39 downto 3);

	-- Signal to control PWM writing
 	signal pwm_write_col_sel	: std_logic_vector(PWM_COL_BITS-1 downto 0);
	signal pwm_write_row_sel	: std_logic_vector(PWM_ROW_BITS-1 downto 0);
	signal pwm_write_data		: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pwm_write_strobe		: std_logic;

	-- Signals to control element memory writing
	signal mem_write_index		: std_logic_vector(INDEX_BITS-1 downto 0);
	signal mem_write_data		: std_logic_vector(63 downto 0);
	signal mem_write_strobe		: std_logic;

	-- Data lines between Control and Memphy
	signal cm_index				: std_logic_vector(INDEX_BITS-1 downto 0);

	-- Data lines between memphy and PWM
	signal mp_elem					: std_logic_vector(PWM_ROW_BITS downto 0);
	--signal mp_index				: std_logic_vector(INDEX_BITS-1 downto 0);

	-- Generate data lines between PWM and adder...
	signal pa_result_data_00 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_01 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_02 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_03 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_04 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_05 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_06 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	signal pa_result_data_07 	: std_logic_vector(RESOLUTION_INPUT-1 downto 0);
	--signal pa_result_index		: std_logic_vector(INDEX_BITS-1 downto 0);

	-- Data lines between adder and requeue...
	signal ar_result_data		: std_logic_vector(RESOLUTION_OUTPUT-1 downto 0);
	--signal ar_result_index		: std_logic_vector(INDEX_BITS-1 downto 0);


begin

   fresp_enable	<= '0';

	control_inst : control
	port map(
		-- Input lines
      in_reset_n => reset_n,
		in_user_clk => user_clk,
      in_user_enable => user_enable,
      in_rt_ready => rt_ready,
      in_qdr_ready => qdr_ready,

		-- Mem read control
 		out_mem_read_enable => mem_read_enable,
		out_mem_read_index => cm_index,

		-- RW Writer control
		out_done => done,
		out_ftr_ptr => ftr_ptr, 	

		-- Memphy init control
		out_write_mem_index => mem_write_index,
		out_write_mem_data => mem_write_data,
		out_write_mem_strobe => mem_write_strobe,

		-- PWM init control
 		out_pwm_write_col_sel => pwm_write_col_sel,
		out_pwm_write_row_sel => pwm_write_row_sel,
		out_pwm_write_data => pwm_write_data,
		out_pwm_write_strobe => pwm_write_strobe,

		-- RT Fabric Request
    	freq_addr => freq_addr,
    	freq_size => freq_size,
    	freq_srctag => freq_srctag,
    	freq_mask => freq_mask,
    	freq_data => freq_data,
    	freq_rw_n => freq_rw_n,
    	freq_ts => freq_ts,
    	freq_valid => freq_valid,
    	freq_enable => freq_enable,

    	-- RT User Response
    	uresp_full => uresp_full,
    	uresp_ts => uresp_ts,
    	uresp_size => uresp_size,
    	uresp_srctag => uresp_srctag,
		uresp_data => uresp_data

		--debug => debug
	);

	memphy_inst : memphy
	port map(
		-- Global signals
		in_user_clk => user_clk,
		in_reset_n => reset_n,

		mem_read_enable => mem_read_enable,

		-- RAM write signals from Control
		in_write_index => mem_write_index,
		in_write_data => mem_write_data,
		in_write_strobe => mem_write_strobe,

		-- SRAM Control and Data
		dr_1		=> dr_1,
		ar_1		=> ar_1,
		aw_1		=> aw_1,
		r_n_1		=> r_n_1,
		w_n_1		=> w_n_1,
		bw_n_1	=> bw_n_1,
		dw_1		=> dw_1,

		dr_2		=> dr_2,
		ar_2		=> ar_2,
		aw_2		=> aw_2,
		r_n_2		=> r_n_2,
		w_n_2		=> w_n_2,
		bw_n_2	=> bw_n_2,
		dw_2		=> dw_2,

		dr_3		=> dr_3,
		ar_3		=> ar_3,
		aw_3		=> aw_3,
		r_n_3		=> r_n_3,
		w_n_3		=> w_n_3,
		bw_n_3	=> bw_n_3,
		dw_3		=> dw_3,

		dr_4		=> dr_4,
		ar_4		=> ar_4,
		aw_4		=> aw_4,
		r_n_4		=> r_n_4,
		w_n_4		=> w_n_4,
		bw_n_4	=> bw_n_4,
		dw_4		=> dw_4,

		-- Read lines
		in_index => cm_index,
		out_data => mp_elem
		--out_index => mp_index

		--debug => debug
	);

	pwm_inst : pwm
	port map(
		-- Global control signal
		in_user_clk => user_clk,
		in_reset_n => reset_n,

		-- PWM write signals
	 	in_reg_col_sel => pwm_write_col_sel,
		in_reg_row_sel => pwm_write_row_sel,
		in_reg_data => pwm_write_data,
		in_reg_strobe => pwm_write_strobe,
		
		-- Input signals
		in_elem => mp_elem,
		--in_index => mp_index,

		-- Output signals
		out_data_00 => pa_result_data_00,
		out_data_01 => pa_result_data_01,
		out_data_02 => pa_result_data_02,
		out_data_03 => pa_result_data_03,
		out_data_04 => pa_result_data_04,
		out_data_05 => pa_result_data_05,
		out_data_06 => pa_result_data_06,
		out_data_07 => pa_result_data_07
		--out_index => pa_result_index

		--debug => debug
	);

	adder_inst : adder
	port map(
		-- Global input signals
		in_user_clk => user_clk,
		in_reset_n => reset_n,

		-- Input signals
		in_data_00 => pa_result_data_00,
		in_data_01 => pa_result_data_01,
		in_data_02 => pa_result_data_02,
		in_data_03 => pa_result_data_03,
		in_data_04 => pa_result_data_04,
		in_data_05 => pa_result_data_05,
		in_data_06 => pa_result_data_06,
		in_data_07 => pa_result_data_07,
		--in_index => pa_result_index,
		
		-- Output (driven) signals
		out_data => ar_result_data
		--out_index => ar_result_index

		--debug => debug
	);
	
	resqueue_inst : resqueue
	port map(
		in_user_clk => user_clk,
		in_reset_n => reset_n,

	 	-- Input signals
		in_data => ar_result_data,
		--in_index => ar_result_index,
		in_index => cm_index,

	  	-- Signals to tell resqueue to write results to RT, and where
		in_done => done,
		in_ftr_ptr => ftr_ptr,

		-- RT user request interface
	   ureq_srctag				=> ureq_srctag,
	   ureq_full				=> ureq_full,
	   ureq_notag				=> ureq_notag,
	   ureq_addr				=> ureq_addr,
	   ureq_size				=> ureq_size,
	   ureq_mask				=> ureq_mask,
	   ureq_data				=> ureq_data,
	   ureq_rw_n				=> ureq_rw_n,
	   ureq_ts					=> ureq_ts,
	   ureq_byte_req			=> ureq_byte_req
		
		--debug => debug	
	);

  
end architecture rtl;
