--------------------------------------------------------------------------------
-- Copyright (c) 1995-2003 Xilinx, Inc.
-- All Right Reserved.
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 7.1.04i
--  \   \         Application : ISE Foundation
--  /   /         Filename : adder_tbw.vhw
-- /___/   /\     Timestamp : Fri Jun 23 05:02:06 2006
-- \   \  /  \ 
--  \___\/\___\ 
--
--Command: 
--Design Name: adder_tbw
--Device: Xilinx
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
library work;
use work.user_pkg.all;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY adder_tbw IS
END adder_tbw;

ARCHITECTURE testbench_arch OF adder_tbw IS
    FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";

    COMPONENT adder
        PORT (
            in_user_clk : In std_logic;
            in_reset_n : In std_logic;
            in_data_00 : In std_logic_vector (31 DownTo 0);
            in_data_01 : In std_logic_vector (31 DownTo 0);
            in_data_02 : In std_logic_vector (31 DownTo 0);
            in_data_03 : In std_logic_vector (31 DownTo 0);
            in_data_04 : In std_logic_vector (31 DownTo 0);
            in_data_05 : In std_logic_vector (31 DownTo 0);
            in_data_06 : In std_logic_vector (31 DownTo 0);
            in_data_07 : In std_logic_vector (31 DownTo 0);
            in_index : In std_logic_vector (25 DownTo 0);
            out_data : Out std_logic_vector (37 DownTo 0);
            out_index : Out std_logic_vector (25 DownTo 0);
            debug : Out std_logic_vector (63 DownTo 0)
        );
    END COMPONENT;

    SIGNAL in_user_clk : std_logic := '0';
    SIGNAL in_reset_n : std_logic := '0';
    SIGNAL in_data_00 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_01 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_02 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_03 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_04 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_05 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_06 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_data_07 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL in_index : std_logic_vector (25 DownTo 0) := "00000000000000000000000000";
    SIGNAL out_data : std_logic_vector (37 DownTo 0) := "00000000000000000000000000000000000000";
    SIGNAL out_index : std_logic_vector (25 DownTo 0) := "00000000000000000000000000";
    SIGNAL debug : std_logic_vector (63 DownTo 0) := "0000000000000000000000000000000000000000000000000000000000000000";

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 200 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : adder
        PORT MAP (
            in_user_clk => in_user_clk,
            in_reset_n => in_reset_n,
            in_data_00 => in_data_00,
            in_data_01 => in_data_01,
            in_data_02 => in_data_02,
            in_data_03 => in_data_03,
            in_data_04 => in_data_04,
            in_data_05 => in_data_05,
            in_data_06 => in_data_06,
            in_data_07 => in_data_07,
            in_index => in_index,
            out_data => out_data,
            out_index => out_index,
            debug => debug
        );

        PROCESS    -- clock process for in_user_clk
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                in_user_clk <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                in_user_clk <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            PROCEDURE CHECK_debug(
                next_debug : std_logic_vector (63 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (debug /= next_debug) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns debug="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, debug);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_debug);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_out_data(
                next_out_data : std_logic_vector (37 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (out_data /= next_out_data) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns out_data="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, out_data);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_out_data);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_out_index(
                next_out_index : std_logic_vector (25 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (out_index /= next_out_index) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns out_index="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, out_index);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_out_index);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            BEGIN
                -- -------------  Current Time:  85ns
                WAIT FOR 85 ns;
                in_reset_n <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  285ns
                WAIT FOR 200 ns;
                in_data_00 <= "00000000000000000000000000000001";
                in_data_01 <= "00000000000000000000000000000010";
                in_data_02 <= "00000000000000000000000000000011";
                in_data_03 <= "00000000000000000000000000000100";
                in_data_04 <= "00000000000000000000000000000101";
                in_data_05 <= "00000000000000000000000000000110";
                in_data_06 <= "00000000000000000000000000000111";
                in_data_07 <= "00000000000000000000000000001000";
                in_index <= "00000000000000001000000000";
                -- -------------------------------------
                -- -------------  Current Time:  485ns
                WAIT FOR 200 ns;
                in_data_00 <= "00000000000000000000000000000010";
                in_data_01 <= "00000000000000000000000000000100";
                in_data_02 <= "00000000000000000000000000000110";
                in_data_03 <= "00000000000000000000000000001000";
                in_data_04 <= "00000000000000000000000000001010";
                in_data_05 <= "00000000000000000000000000001100";
                in_data_06 <= "00000000000000000000000000001110";
                in_data_07 <= "00000000000000000000000000010000";
                in_index <= "00010000101111101011100011";
                -- -------------------------------------
                -- -------------  Current Time:  685ns
                WAIT FOR 200 ns;
                in_data_00 <= "00000000000000000000000000000000";
                in_data_01 <= "00000000000000000000000000000000";
                in_data_02 <= "00000000000000000000000000000000";
                in_data_03 <= "00000000000000000000000000000000";
                in_data_04 <= "00000000000000000000000000000000";
                in_data_05 <= "00000000000000000000000000000000";
                in_data_06 <= "00000000000000000000000000000000";
                in_data_07 <= "00000000000000000000000000000000";
                in_index <= "00000000000000000000000000";
                -- -------------------------------------
                -- -------------  Current Time:  915ns
                WAIT FOR 230 ns;
                CHECK_out_data("00000000000000000000000000000000100100", 915);
                CHECK_out_index("00000000000000001000000000", 915);
                -- -------------------------------------
                -- -------------  Current Time:  1115ns
                WAIT FOR 200 ns;
                CHECK_out_data("00000000000000000000000000000001001000", 1115);
                CHECK_out_index("00010000101111101011100011", 1115);
                -- -------------------------------------
                -- -------------  Current Time:  1315ns
                WAIT FOR 200 ns;
                CHECK_out_data("00000000000000000000000000000000000000", 1315);
                CHECK_out_index("00000000000000000000000000", 1315);
                -- -------------------------------------
                WAIT FOR 885 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

