import javax.swing.*;
import java.awt.*;
import javax.swing.plaf.*;
import java.io.File;
import java.awt.event.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import java.lang.Thread;

public class FPWMSim extends JFrame implements Runnable, KeyListener {
    private File inputfile, outputfile;
    private PrefManager pm;
    private boolean generateRandom;
	private List<FPWM_Module> modules;
	private int step;
	private Thread masterThread;

	private boolean do_step, do_freeze;
	private int sleep_time;

    public FPWMSim(File inputfile, File outputfile, boolean generateRandom) {
		super("FPWMSim");

		this.getContentPane().setLayout(new GridLayout(5,1));
		this.setBackground(Color.white);
		this.setSize(1110,1000);

		this.addWindowListener(
		  new WindowAdapter() {
			public void windowClosing(WindowEvent event){
				System.exit(0);
			}
		  }
		);

		this.addKeyListener(this);

		pm = new PrefManager(inputfile);

        initModules();

		this.setVisible(true);

		step = 0;

		sleep_time = 1000;
		do_freeze = true;
		do_step = false;

        masterThread = new Thread(this);
        masterThread.start();
    }

	public void paint(Graphics g) {
		for(FPWM_Module m : modules ) {
			m.repaint();
		}
	}

	public Dimension getPreferredSize(){
		return new Dimension(1300,1000);
	}

	private void initModules() {
		modules = Collections.checkedList(new ArrayList<FPWM_Module>(), FPWM_Module.class);

		modules.add(new FPWM_Module_Mem(this, pm));
		modules.add(new FPWM_Module_PWM(this, pm));
		modules.add(new FPWM_Module_Adder(this, pm));
		modules.add(new FPWM_Module_Result(this, pm));
		modules.add(new FPWM_Module_Output(this, pm));
	}


	// Runnable methods

	public void run() {
		while(true) {
			System.out.println("Step " + ++step + "...");

			FPWM_DataPackage dataPackage = null;

			for(FPWM_Module m : modules ) {
				dataPackage = m.updateDataState(dataPackage);
			}

			repaint();

			try {
				do {
					if(do_freeze) {
						do_step = false;

						while(do_freeze && ! do_step) {
							masterThread.sleep(10);
						}
					}
					else {
						masterThread.sleep(sleep_time);
					}
				} while (do_freeze && ! do_step);
			} catch(java.lang.InterruptedException ie) {}
		}
	}


	public void speed_increase() {
		if(sleep_time > 0)
			sleep_time -= 50;

		repaint();
	}

	public void speed_decrease() {
		sleep_time += 50;
		repaint();
	}

	public void speed_toggle_freeze() {
		do_freeze = ! do_freeze;
		repaint();
	}

	public void speed_step() {
		do_step = true;
		repaint();
	}

	public int get_sleep_time() {
		return do_freeze ? -1 : sleep_time;
	}

	// KeyListener methods

	public void keyTyped(KeyEvent e) {
		return;
	}

	public void keyPressed(KeyEvent e) {
		int keyCode = e.getKeyCode();

		switch(keyCode) {
			case KeyEvent.VK_UP:	speed_increase();
									break;

			case KeyEvent.VK_DOWN:	speed_decrease();
									break;

			case KeyEvent.VK_RIGHT:	speed_step();
									break;

			case KeyEvent.VK_LEFT:	speed_toggle_freeze();
									break;

			default:				break;
		}
	}

	public void keyReleased(KeyEvent e) {
		return;
	}


	public static void usage() {
		System.out.println("java FPWIMSim [ -genrandom | -load datafile]");
		System.exit(-1);
	}

    public static void main(String[] args){
		boolean generateRandom = false;
		File inputfile = new File("data.in");
		File outputfile = new File("data.out");

		if(args.length > 0) {
			if(args[0].equalsIgnoreCase("-genrandom")) {
				generateRandom = true;
				inputfile = new File("rdata.in");
			}
			else if(args[0].equalsIgnoreCase("-indata")) {
				if(args.length > 1) {
					inputfile = new File(args[1]);
				}
				else {
					usage();
				}
			}
			else {
				usage();
			}
		}

        FPWMSim fs = new FPWMSim(inputfile, outputfile, generateRandom);
    }
}

