import java.awt.*;
import javax.swing.*;

public class FPWM_Module_Adder extends FPWM_Module {

	long[] input;
	long[][] sum;
	int[] index;

	int in_index;

	int pwmCols, pwmRows, sumRows;

	public FPWM_Module_Adder(FPWMSim main, PrefManager pm) {
		super(main, pm);

		pwmCols = pm.getPWMCols();
		pwmRows = pm.getPWMRows();
		sumRows = (int)Math.round(Math.log(pwmCols)/Math.log(2))+1;

		sum = new long[sumRows][pwmCols];
		input = new long[pwmCols];
		index = new int[sumRows];

		in_index = -1;


		java.util.Arrays.fill(index, -1);

		System.out.println("Created an Adder");
	}

	public FPWM_DataPackage updateDataState(FPWM_DataPackage dataPackage) {

		// parse package

		FPWM_DataPackage_PA dppa = null;

		if(dataPackage instanceof FPWM_DataPackage_PA) {
			dppa = (FPWM_DataPackage_PA) dataPackage;
		}
		else {
			System.out.println("Error: FPWM_Module_Adder::updateDataState expects a FPWM_DataPackage_PA");
		}

		// update data

		for(int i=0; i<sumRows-1; i++) {
			index[i] = index[i+1];
		}

		index[sumRows-1] = in_index;


		int step = pwmCols;

		for(int i = sumRows-1; i>0; i--) {
			for(int j=0; j<pwmCols-step+1; j+=step) {
				sum[i][j] = sum[i-1][j] + sum[i-1][j+step/2];
			}

			step /= 2;
		}

		for(int i=0; i<pwmCols; i+=1) {
			sum[0][i] = input[i];
		}

		input = dppa.getPwmScores().clone();
		in_index = dppa.getIndex();

		return new FPWM_DataPackage_AR(index[0], sum[sumRows-1][0]);
	}

	public void do_paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0,0,1100,250);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 16));
		g.drawString("Adder", 10, 20);
		g.drawString("Module", 10, 40);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.PLAIN, 12));


		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.setColor(Color.BLACK);

		int xoffset = 100;
		int yoffset = 10;

		// draw input buffer

		int xoff = xoffset;
		int yoff = yoffset;

		for(int c=0; c<pwmCols; c++) {
			g.drawRect(xoff, yoff, 20, 20);
			g.drawString(input[c]+"", xoff+6, yoff+15);

			g.drawLine(xoff+10, yoff-100, xoff+10, yoff-2);
			g.drawLine(xoff+10, yoff-2, xoff+8, yoff-4);
			g.drawLine(xoff+10, yoff-2, xoff+12, yoff-4);

			xoff += 30;
		}

		// draw sum matrix

		int xoffb = 0;
		xoff = xoffset;
		yoff = yoffset + 35;

		int skip = 1;

		for(int i=0; i<sumRows; i++) {

			for(int j=0; j<pwmCols; j+=skip) {
				g.drawString(sum[i][j]+"", xoff+6, yoff+15);

				if(i==sumRows-1) {
					g.setColor(sum[i][j]==0 ? Color.GRAY : Color.RED);
					g.drawRect(xoff-40, yoff, 100, 20);
					g.drawLine(xoff+62, yoff+9, xoff+585, yoff+9);
					g.drawLine(xoff+585, yoff+9, xoff+585, yoffset+150);

					g.drawLine(xoff+585, yoffset+148, xoff+583, yoffset+146);
					g.drawLine(xoff+585, yoffset+148, xoff+587, yoffset+146);

					g.drawRect(xoff+560, yoffset+150, 50, 20);
					g.drawString(sum[i][j]+"", xoff+570, yoffset+165);

					g.drawLine(xoff+585, yoffset+172, xoff+585, yoff+200);
				}

				xoff += 30*skip;
			}

			xoffb = xoffb*2 + 15;
			xoff = xoffset+xoffb;
			yoff+= 25;
			skip *= 2;
		}



		// draw index buffer

		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.setColor(Color.BLACK);

		xoff = xoffset+730;
		yoff = yoffset+10;

		for(int i=sumRows-1; i>=0; i--) {
			g.setColor(index[i] >= 0 ? Color.RED : Color.GRAY);

			if(i==sumRows-1) {
				g.drawLine(xoff+25, yoff-100, xoff+25, yoff-2);
				g.drawLine(xoff+25, yoff-2, xoff+27, yoff-4);
				g.drawLine(xoff+25, yoff-2, xoff+23, yoff-4);
			}

			g.drawLine(xoff+25, yoff+22, xoff+25, yoff+28);
			g.drawLine(xoff+25, yoff+28, xoff+27, yoff+26);
			g.drawLine(xoff+25, yoff+28, xoff+23, yoff+26);

			g.drawRect(xoff, yoff, 50, 20);
			g.drawString(index[i]+"", xoff + 7, yoff+15);

			if(i==0) {
				g.drawLine(xoff+25, yoff+22, xoff+25, yoff+100);
			}

			yoff += 30;
		}

		drawBorder(g);

	}
}