import java.awt.*;
import javax.swing.*;

public class FPWM_Module_Mem extends FPWM_Module {

	int[] seq;
	int[] seq_display;

	int seq_pointer;
	int elemCount;

	double scaling;

	int current_element, current_index;

	public FPWM_Module_Mem(FPWMSim main, PrefManager pm) {
		super(main, pm);

		seq = pm.getSeq();
		elemCount = pm.getSeqLength();
		seq_pointer = 0;

		scaling = 1;

		seq_display = new int[Math.min(38, MEMORY_FETCH_DELAY + elemCount)];
		java.util.Arrays.fill(seq_display, -1);

		System.out.println("Created a Mem");
	}

	public FPWM_DataPackage updateDataState(FPWM_DataPackage dataPackage) {
		// The memory module doesn't use the incoming dataPackage in any way.

		// setup display

		int ii = seq_pointer;
		int oi = 0;

		while(ii++ < MEMORY_FETCH_DELAY) {
			seq_display[oi++] = -1;
		}

		while(oi < seq_display.length && ii < seq.length + MEMORY_FETCH_DELAY) {
			seq_display[oi++] = seq[ii++ - MEMORY_FETCH_DELAY];
		}

		while(oi < seq_display.length) {
			seq_display[oi++] = -1;
		}

		current_element = seq_display[0];
		current_index = seq_pointer-MEMORY_FETCH_DELAY;


		seq_pointer++;

		return new FPWM_DataPackage_MP(current_element == -1 ? -1 : current_index, current_element);
	}

	public void do_paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0,0,1100,250);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 16));
		g.drawString("Memory", 10, 20);
		g.drawString("Module", 10, 40);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.PLAIN, 12));


		// draw memory grid

		int xoffset = 100;
		int yoffset = 70;

		g.drawString("Out", xoffset, yoffset-15);

		g.drawLine(xoffset+33, yoffset-20, xoffset+68, yoffset-20);
		g.drawLine(xoffset+33, yoffset-20, xoffset+33, yoffset-15);
		g.drawString("Memory Fetch Delay", xoffset+73, yoffset-15);
		g.drawLine(xoffset+193, yoffset-20, xoffset+228, yoffset-20);
		g.drawLine(xoffset+228, yoffset-20, xoffset+228, yoffset-15);

		g.drawLine(xoffset+240, yoffset-20, xoffset+537, yoffset-20);
		g.drawLine(xoffset+240, yoffset-20, xoffset+240, yoffset-15);
		g.drawString("Future Memory Elements", xoffset+542, yoffset-15);
		g.drawLine(xoffset+682, yoffset-20, xoffset+979, yoffset-20);
		g.drawLine(xoffset+979, yoffset-20, xoffset+979, yoffset-15);

		for(int i=0; i<elemCount+1; i++) {
			if(seq_display[i] == -1 || i == elemCount) {
				g.setColor(Color.GRAY);
			}
			else if(i==0) {
				g.setColor(Color.RED);
			}
			else if(i < 8) {
				g.setColor(Color.GREEN);
			}
			else {
				g.setColor(Color.BLACK);
			}

			int drawOffset = (i * 30) + xoffset;

			if(i == elemCount) {
				g.drawRect(drawOffset, yoffset, 20, 20);
				g.drawString("...", drawOffset+7, yoffset+15);
			}
			else {
				g.drawRect(drawOffset, yoffset, 20, 20);
				g.drawString(seq_display[i]+"", drawOffset+7, yoffset+15);

				// make a cute little arrow

				g.setColor(Color.BLACK);
				g.drawLine(drawOffset+22, yoffset+10, drawOffset+28, yoffset+10);
				g.drawLine(drawOffset+22, yoffset+10, drawOffset+24, yoffset+12);
				g.drawLine(drawOffset+22, yoffset+10, drawOffset+24, yoffset+8);
			}
		}

		// make the output stuff

		g.setColor(current_element == -1 ? Color.GRAY : Color.RED);

		g.drawLine(xoffset+711, yoffset+52, xoffset+711, yoffset+200);
		g.drawLine(xoffset+755, yoffset+52, xoffset+755, yoffset+200);

		g.drawLine(xoffset+10, yoffset+20, xoffset+10, yoffset+40);
		g.drawLine(xoffset+10, yoffset+40, xoffset+700, yoffset+40);

		g.setFont(new Font("Arial", Font.BOLD, 16));

		g.drawRect(xoffset+700, yoffset+30, 22, 20);
		g.drawString(current_element+"", xoffset+707, yoffset+46);

		g.drawRect(xoffset+730, yoffset+30, 50, 20);
		g.drawString(current_index+"", xoffset+737, yoffset+46);

		if(current_element == -1) {
			g.drawString("(NULL element)", xoffset+790, yoffset+46);
		}


		// draw border

		drawBorder(g);
	}






}