import java.awt.*;
import javax.swing.*;

public class FPWM_Module_Output extends FPWM_Module {

	int lastOutputIndex;
	long lastOutputData;
	boolean changing;

	int loadPWMCycles, loadSEQCycles, computeCycles, outputCycles, totalCycles;

	public FPWM_Module_Output(FPWMSim main, PrefManager pm) {
		super(main, pm);

		int pwmCols = pm.getPWMCols();
		int pwmRows = pm.getPWMRows();
		int seqLength = pm.getSeqLength();

		loadPWMCycles = (int)(pwmCols * pwmRows * 9)/8;
		loadSEQCycles = (int)((seqLength*9)/64);
		computeCycles = seqLength + MEMORY_FETCH_DELAY + (int)(Math.round(Math.log(pwmCols)/Math.log(2))) + 1;
		outputCycles = (RESULT_QUEUE_LENGTH*9)/8 + 1;
		totalCycles = loadPWMCycles + loadSEQCycles + computeCycles + outputCycles;

		System.out.println("Created an Output");
	}

	public FPWM_DataPackage updateDataState(FPWM_DataPackage dataPackage) {
		// parse package

		FPWM_DataPackage_RO dpro = null;

		if(dataPackage == null) {
			changing = false;
			dpro = null;
		}
		else if(dataPackage instanceof FPWM_DataPackage_RO) {
			dpro = (FPWM_DataPackage_RO) dataPackage;
		}
		else {
			System.out.println("Error: FPWM_Module_Output::updateDataState expects a FPWM_DataPackage_RO");
		}

		if(dpro != null) {
			changing = ! (dpro.getIndex() == lastOutputIndex && dpro.getData() == lastOutputData);

			lastOutputIndex = dpro.getIndex();
			lastOutputData = dpro.getData();
		}

		return null;
	}

	public void do_paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0,0,1100,250);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 16));
		g.drawString("Output", 10, 20);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.PLAIN, 12));

		int xoffset = 100;
		int yoffset = 10;

		// est compute window

		g.drawRect(xoffset, yoffset, 200, 150);
		g.drawString("Estimated Compute Time", xoffset+10, yoffset+15);
		g.drawLine(xoffset, yoffset+20, xoffset+200, yoffset+20);

		g.drawString("PWM Load Cycles", xoffset+10, yoffset+40);
		g.drawString(loadPWMCycles+"", xoffset+150, yoffset+40);

		g.drawString("SEQ Load Cycles", xoffset+10, yoffset+60);
		g.drawString(loadSEQCycles+"", xoffset+150, yoffset+60);

		g.drawString("Compute Cycles", xoffset+10, yoffset+80);
		g.drawString(computeCycles+"", xoffset+150, yoffset+80);

		g.drawString("Output Cycles", xoffset+10, yoffset+100);
		g.drawString(outputCycles+"", xoffset+150, yoffset+100);

		g.drawString("Total Cycles", xoffset+10, yoffset+130);
		g.drawString(totalCycles+"", xoffset+150, yoffset+130);


		// draw speed stuff

		String speed_string = null;
		int sleep_time = main.get_sleep_time();

		switch(sleep_time ) {
			case 0 : speed_string = "MAX"; break;
			case -1: speed_string = "PAUSED"; break;
			default: speed_string = sleep_time + " millisec delay"; break;
		}

		g.drawString("Current Simulation Speed: " + speed_string, xoffset+220, yoffset+15);

		g.drawString("Controls: ", xoffset+220, yoffset+40);
		g.drawString("Arrow Key Up:", xoffset+280, yoffset+40);
		g.drawString("Increase Speed", xoffset+380, yoffset+40);
		g.drawString("Arrow Key Down:", xoffset+280, yoffset+60);
		g.drawString("Decrease Speed", xoffset+380, yoffset+60);
		g.drawString("Arrow Key Left:", xoffset+280, yoffset+80);
		g.drawString("Pause", xoffset+380, yoffset+80);
		g.drawString("Arrow Key Right:", xoffset+280, yoffset+100);
		g.drawString("Step (when Paused)", xoffset+380, yoffset+100);



		// draw entry stuff

		g.setColor(changing ? Color.RED : Color.GRAY);

		//g.drawLine(xoffset+755, 0, xoffset+755, 18);
		//g.drawLine(xoffset+755, 18, xoffset+753, 16);
		//g.drawLine(xoffset+755, 18, xoffset+757, 16);

		g.drawString("Most Recent Output:", 630, 35);

		g.drawRect(xoffset+730, 20, 50, 20);
		g.drawString(lastOutputIndex+"", xoffset+736, 35);

		//g.drawLine(xoffset+690, 0, xoffset+690, 18);
		//g.drawLine(xoffset+690, 18, xoffset+692, 16);
		//g.drawLine(xoffset+690, 18, xoffset+688, 16);
		g.drawRect(xoffset+655, 20, 50, 20);
		g.drawString(lastOutputData+"", xoffset+661, 35);



		drawBorder(g);

	}






}