import java.awt.*;
import javax.swing.*;

public class FPWM_Module_PWM extends FPWM_Module {

	long[][] pwm;
	int pwmCols, pwmRows;

	int[] window;
	int[] indexes;
	long[] output;

	boolean active;

	int in_index, in_element;

	public FPWM_Module_PWM(FPWMSim main, PrefManager pm) {
		super(main, pm);

		pwm = pm.getPWM();
		pwmCols = pm.getPWMCols();
		pwmRows = pm.getPWMRows();

		window = new int[pwmCols];
		indexes = new int[pwmCols];

		output = new long[pwmCols];

		java.util.Arrays.fill(window, -1);
		java.util.Arrays.fill(indexes, -1);
		java.util.Arrays.fill(output, 0);

		in_index = -1;
		in_element = -1;

		active = false;

		System.out.println("Created a PWM");
	}

	public FPWM_DataPackage updateDataState(FPWM_DataPackage dataPackage) {
		FPWM_DataPackage_MP dpmp = null;

		if(dataPackage instanceof FPWM_DataPackage_MP) {
			dpmp = (FPWM_DataPackage_MP) dataPackage;
		}
		else {
			System.out.println("Error: FPWM_Module_PWM::updateDataState expects a FPWM_DataPackage_MP");
		}

		int i = 0;

		for(i=0; i<pwmCols-1; i++) {
			window[i] = window[i+1];
			indexes[i] = indexes[i+1];
		}

		window[i] = in_element;
		indexes[i] = in_index;

		active = true;

		for(i=0; i<pwmCols; i++) {
			if(window[i] == -1) {
				active = false;
			}
		}

		for(int c=0; c<pwmCols; c++) {
			output[c] = (window[c] < 0 || !active) ? 0 : pwm[window[c]][c];
		}

		// delayed one cycle from when they're accepted
		in_index = dpmp.getIndex();
		in_element = dpmp.getElement();

		return new FPWM_DataPackage_PA(indexes[0], output);
	}

	public void do_paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0,0,1100,250);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 16));
		g.drawString("PWM", 10, 20);
		g.drawString("Module", 10, 40);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.PLAIN, 12));

		int xoffset = 100;
		int yoffset = 10;

		g.setColor(in_element == -1 ? Color.GRAY : Color.RED);

		g.drawLine(xoffset+711, yoffset-60, xoffset+711, yoffset-2);
		g.drawLine(xoffset+711, yoffset-2, xoffset+713, yoffset-4);
		g.drawLine(xoffset+711, yoffset-2, xoffset+709, yoffset-4);

		g.drawLine(xoffset+755, yoffset-60, xoffset+755, yoffset-2);
		g.drawLine(xoffset+755, yoffset-2, xoffset+757, yoffset-4);
		g.drawLine(xoffset+755, yoffset-2, xoffset+753, yoffset-4);

		g.setFont(new Font("Arial", Font.BOLD, 16));

		g.drawRect(xoffset+700, yoffset, 22, 20);
		g.drawString(in_element+"", xoffset+707, yoffset+15);

		g.drawRect(xoffset+730, yoffset, 50, 20);
		g.drawString(in_index+"", xoffset+737, yoffset+15);

		if(in_element == -1) {
			g.drawString("(NULL element)", xoffset+790, yoffset+15);
		}

		g.drawLine(xoffset+755, yoffset+22, xoffset+755, yoffset+28);
		g.drawLine(xoffset+755, yoffset+28, xoffset+753, yoffset+26);
		g.drawLine(xoffset+755, yoffset+28, xoffset+757, yoffset+26);


		g.setFont(new Font("Arial", Font.PLAIN, 12));

		// draw window
		for(int i=0; i<pwmCols; i++) {
			if(window[i] == -1) {
				g.setColor(Color.GRAY);
			}
			else {
				if(active) {
					g.setColor(Color.RED);
				}
				else {
					g.setColor(Color.GREEN);
				}
			}

			int drawOffset = (i * 30) + xoffset;

			g.drawRect(drawOffset, yoffset, 20, 20);
			g.drawString(window[i]+"", drawOffset+6, yoffset+15);

			// make a cute little arrow

			g.setColor(Color.BLACK);
			g.drawLine(drawOffset+22, yoffset+10, drawOffset+28, yoffset+10);
			g.drawLine(drawOffset+22, yoffset+10, drawOffset+24, yoffset+12);
			g.drawLine(drawOffset+22, yoffset+10, drawOffset+24, yoffset+8);


			// draw line from entry point to window[max]
			if(i == pwmCols-1) {
				g.drawLine(drawOffset+22, yoffset+10, xoffset+698, yoffset+10);
			}
		}


		// draw index buffer

		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.setColor(Color.BLACK);

		int index_yoff = yoffset+30;

		for(int i=0; i<pwmCols; i += 4) {
			int startx = xoffset+730;

			for(int j=0; j<4; j++) {
				g.setColor(indexes[pwmCols - i - j - 1] >= 0 ? Color.RED : Color.GRAY);

				if(j < 3) {
					g.drawLine(startx + 52 + j*60, index_yoff+10, startx + 58 + j*60, index_yoff+10);
					g.drawLine(startx + 58 + j*60, index_yoff+10, startx + 56 + j*60, index_yoff+12);
					g.drawLine(startx + 58 + j*60, index_yoff+10, startx + 56 + j*60, index_yoff+8);
				}

				g.drawRect(startx + j*60, index_yoff, 50, 20);
				g.drawString(indexes[pwmCols - i - j - 1]+"", startx + j*60 + 7, index_yoff+15);
			}

			g.drawLine(startx + 52 + 3*60, index_yoff+10, startx + 58 + 3*60, index_yoff+10);
			g.drawLine(startx + 58 + 3*60, index_yoff+10, startx + 58 + 3*60, index_yoff+24);
			g.drawLine(startx + 58 + 3*60, index_yoff+24, startx + 25, index_yoff+24);

			g.drawLine(startx + 25, index_yoff+24, startx + 25, index_yoff+29);
			g.drawLine(startx + 25, index_yoff+29, startx + 23, index_yoff+27);
			g.drawLine(startx + 25, index_yoff+29, startx + 27, index_yoff+27);

			index_yoff += 30;
		}

		g.drawLine(xoffset+755, index_yoff, xoffset+755, index_yoff+100);

		// Draw the PWM

		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.setColor(Color.BLACK);

		g.drawLine(xoffset, yoffset+26, xoffset+pwmCols*30-10, yoffset+26);
		g.drawLine(xoffset, yoffset+26+pwmRows*30, xoffset+pwmCols*30-10, yoffset+26+pwmRows*30);

		int xoff = xoffset;
		int yoff = yoffset+31;



		for(int r=0; r<pwmRows; r++) {
			xoff = xoffset;

			for(int c=0; c<pwmCols; c++) {
				g.setColor(window[c] == r && active ? Color.RED : Color.BLACK);

				g.drawRect(xoff, yoff, 20, 20);
				g.drawString(pwm[r][c]+"", xoff+6, yoff+15);

				xoff += 30;
			}

			yoff += 30;
		}


		// Draw the output buffer

		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.setColor(Color.BLACK);

		xoff = xoffset;
		yoff = yoffset+30+pwmRows*30;

		for(int c=0; c<pwmCols; c++) {
			g.drawRect(xoff, yoff, 20, 20);
			g.drawString(output[c]+"", xoff+6, yoff+15);
			g.drawLine(xoff+10, yoff+20, xoff+10, yoff+100);

			xoff += 30;
		}

		drawBorder(g);

	}






}