import java.awt.*;
import javax.swing.*;

public class FPWM_Module_Result extends FPWM_Module {

	long in_data, out_data;
	int in_index, out_index;
	int entrypoint, exitpoint;
	int outputTrigger, outputIndex;

	long[] result_data;
	int[] result_index;

	int resqueue_length;

	public FPWM_Module_Result(FPWMSim main, PrefManager pm) {
		super(main, pm);

		in_data = 0;
		in_index = -1;
		entrypoint = -1;
		exitpoint = -1;

		resqueue_length = RESULT_QUEUE_LENGTH;

		outputTrigger = pm.getSeqLength() - pm.getPWMRows() - MEMORY_FETCH_DELAY + (int)Math.round(Math.log(pm.getPWMCols())/Math.log(2))+1;
		outputIndex = resqueue_length-1;

		result_data = new long[resqueue_length];
		result_index = new int[resqueue_length];

		java.util.Arrays.fill(result_data, -1);
		java.util.Arrays.fill(result_index, -1);

		System.out.println("Created a Result");
	}

	public FPWM_DataPackage updateDataState(FPWM_DataPackage dataPackage) {

		// parse package

		FPWM_DataPackage_AR dpar = null;

		if(dataPackage instanceof FPWM_DataPackage_AR) {
			dpar = (FPWM_DataPackage_AR) dataPackage;
		}
		else {
			System.out.println("Error: FPWM_Module_Result::updateDataState expects a FPWM_DataPackage_AR");
		}

		entrypoint = -1;

		if(in_index>=0) {

			// find entry point

			for(int i=resqueue_length-1; i>=0; i--) {
				if(in_data > result_data[i]) {
					entrypoint = i;
					break;
				}
			}
		}

		//System.out.println("Data=" + in_data + ", Index=" + in_index + ", Entrypoint=" + entrypoint);

		if(entrypoint>=0) {
			// shift results

			for(int i=0; i<entrypoint; i++) {
				//System.out.print("Moving from " + (i+1) + " to " + i);
				result_data[i] = result_data[i+1];
				result_index[i] = result_index[i+1];
			}

			// enter new result

			result_data[entrypoint] = in_data;
			result_index[entrypoint] = in_index;
		}

		in_data = dpar.getData();
		in_index = dpar.getIndex();

		if((in_index > outputTrigger || outputIndex != resqueue_length-1) && outputIndex >= 0) {
			exitpoint = outputIndex;
			out_index = result_index[outputIndex];
			out_data = result_data[outputIndex--];
			return new FPWM_DataPackage_RO(out_index, out_data);
		}
		else {
			exitpoint = -1;
			out_index = 0;
			out_data = 0;
			return null;
		}
	}

	public void do_paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0,0,1100,250);

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 16));
		g.drawString("Result", 10, 20);
		g.drawString("Module", 10, 40);

		int xoffset = 100;
		int yoffset = 75;

		int xoff = xoffset;
		int yoff = yoffset;

		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.PLAIN, 12));
		g.drawString("Index", xoff+6, yoff+15);
		g.drawString("Result", xoff+6, yoff+45);

		// draw entry stuff

		g.setColor(in_index>=0 ? Color.RED : Color.GRAY);

		g.drawLine(xoffset+755, 0, xoffset+755, 18);
		g.drawLine(xoffset+755, 18, xoffset+753, 16);
		g.drawLine(xoffset+755, 18, xoffset+757, 16);
		g.drawRect(xoffset+730, 20, 50, 20);
		g.drawString(in_index+"", xoffset+736, 35);

		g.drawLine(xoffset+690, 0, xoffset+690, 18);
		g.drawLine(xoffset+690, 18, xoffset+692, 16);
		g.drawLine(xoffset+690, 18, xoffset+688, 16);
		g.drawRect(xoffset+665, 20, 50, 20);
		g.drawString(in_data+"", xoffset+671, 35);

		// draw entrypointer

		if(in_index < 0)
			g.setColor(Color.GRAY);
		else if(entrypoint < 0)
			g.setColor(Color.BLACK);
		else
			g.setColor(Color.RED);

		int insert_to = xoffset + 85 + resqueue_length*60 - (resqueue_length - entrypoint)*60;

		g.drawLine(xoffset+755, 42, xoffset+755, 50);
		g.drawLine(xoffset+680, 42, xoffset+680, 50);
		g.drawLine(xoffset+755, 50, insert_to, 50);

		if(entrypoint < 0) {
			g.drawLine(insert_to, 50, insert_to+2, 52);
			g.drawLine(insert_to, 50, insert_to+2, 48);
		}
		else {
			g.drawLine(insert_to, 50, insert_to, 60);
			g.drawLine(insert_to, 60, insert_to+2, 58);
			g.drawLine(insert_to, 60, insert_to-2, 58);
		}

		// draw exitpoint

		if(exitpoint < 0)
			g.setColor(Color.GRAY);
		else
			g.setColor(Color.RED);

		insert_to = xoffset + 85 + resqueue_length*60 - (resqueue_length - exitpoint)*60;

		g.drawLine(xoffset+755, 158, xoffset+755, 150);
		g.drawLine(xoffset+755, 158, xoffset+753, 156);
		g.drawLine(xoffset+755, 158, xoffset+757, 156);

		g.drawLine(xoffset+680, 158, xoffset+680, 150);
		g.drawLine(xoffset+680, 158, xoffset+678, 156);
		g.drawLine(xoffset+680, 158, xoffset+682, 156);


		g.drawLine(xoffset+755, 150, insert_to, 150);

		g.drawRect(xoffset+655, 160, 50, 20);
		g.drawString(out_data+"",xoffset+661, 175);

		g.drawRect(xoffset+730, 160, 50, 20);
		g.drawString(out_index+"",xoffset+736, 175);

		if(exitpoint >= 0) {
			g.drawLine(insert_to, 150, insert_to, 140);
		}

		// draw queue

		for(int i=0; i<resqueue_length; i++) {
			g.setColor(i <= entrypoint ? Color.RED : Color.BLACK);

			xoff += 60;

			g.drawRect(xoff, yoff, 50, 20);
			g.drawString(result_index[i]+"", xoff+6, yoff+15);

			g.drawRect(xoff, yoff+30, 50, 20);
			g.drawString(result_data[i]+"", xoff+6, yoff+45);

			g.drawLine(xoff, yoff+10, xoff-8, yoff+10);
			g.drawLine(xoff-8, yoff+10, xoff-6, yoff+8);
			g.drawLine(xoff-8, yoff+10, xoff-6, yoff+12);

			g.drawLine(xoff, yoff+40, xoff-8, yoff+40);
			g.drawLine(xoff-8, yoff+40, xoff-6, yoff+38);
			g.drawLine(xoff-8, yoff+40, xoff-6, yoff+42);
		}


		drawBorder(g);
	}
}