import java.io.*;

public class PrefManager {

	long[][] pwm;
	int[] seq;
	int pwmCols, pwmRows, elemCount;

	public PrefManager(File inputfile) {
		if(inputfile.exists()) {
			try {
				FileInputStream fis = new FileInputStream(inputfile);
				InputStreamReader isr = new InputStreamReader(fis);
				BufferedReader fbr = new BufferedReader(isr);

				String b = null;

				while((b = fbr.readLine()) != null) {
					b = b.trim();

					if(b.startsWith("#") || b.startsWith("!") || b.startsWith("//") || b.startsWith("-")) {
						continue;
					}

					if(b.startsWith("PWM")) {
						String[] arg = b.split(",");

						pwmCols = Integer.parseInt(arg[1].trim());
						pwmRows = Integer.parseInt(arg[2].trim());

						int pwm_ri = 0;
						int pwm_ci = 0;

						pwm = new long[pwmRows][pwmCols];

						String b2 = null;

						System.out.println("\n\nParsing PWM (expecting " + pwmCols + "x" + pwmRows + " elements)");

						while((b2 = fbr.readLine()) != null) {
							b2 = b2.trim();
							if(b2.startsWith("/PWM")) {
								break;
							}
							else {
								String[] arg2 = b2.split(",");
								for(String a : arg2) {
									String temp = a.trim();

									if(temp.length() > 0) {
										long pe = Long.parseLong(temp);
										pwm[pwm_ri][pwm_ci++] = pe;
										System.out.print(pe+"\t");
									}
								}
							}

							pwm_ri++;
							pwm_ci = 0;
							System.out.print("\n");
						}
					}
					else if(b.startsWith("SEQ")) {
						String[] arg = b.split(",");
						elemCount = Integer.parseInt(arg[1].trim());

						seq = new int[elemCount];

						int i = 0;

						String b2 = null;

						System.out.println("\n\nParsing SEQ (expecting "+ elemCount +" elements)");

						while((b2 = fbr.readLine()) != null) {
							b2 = b2.trim();
							if(b2.startsWith("/SEQ")) {
								break;
							}
							else {
								String[] arg2 = b2.split(",");
								for(String a : arg2) {
									String temp = a.trim();

									if(temp.length() > 0) {
										int se = Integer.parseInt(temp);
										seq[i++] = se;
										System.out.print(se+" ");
									}
								}
							}
						}
					}
				}
			}
			catch(Exception e) {
				e.printStackTrace();
				System.exit(-1);
			}
		}
		else {
			System.out.println("Error: File " + inputfile.getAbsolutePath() + " does not exist.");
			System.exit(-1);
		}
	}

	public long[][] getPWM() {
		return pwm;
	}


	public int getPWMRows() {
		return pwmRows;
	}

	public int getPWMCols() {
		return pwmCols;
	}

	public int[] getSeq() {
		return seq;
	}

	public int getSeqLength() {
		return elemCount;
	}

}